package org.infinispan.quickstart.securecache.rest;

import java.io.IOException;

import org.codehaus.jackson.JsonGenerator;
import org.codehaus.jackson.JsonProcessingException;
import org.codehaus.jackson.map.JsonSerializer;
import org.codehaus.jackson.map.SerializerProvider;
import org.codehaus.jackson.map.annotate.JsonSerialize;

@JsonSerialize
public class CacheEntry<K extends Comparable<K>,V extends Comparable<V>> implements Comparable<CacheEntry<K,V>> {
	public K getKey() {
		return key;
	}

	public V getValue() {
		return value;
	}

	private K key; 
	private V value; 
	
	public CacheEntry(K key, V value) {
		this.key = key;
		this.value = value;
	}

	@Override
	public int compareTo(CacheEntry<K, V> o) {
		return this.key.compareTo(o.key);
	}
	
	
}

class CacheEntrySerializer<K extends Comparable<K>,V extends Comparable<V>> extends JsonSerializer<CacheEntry<K,V>> {

	@Override
	public void serialize(CacheEntry<K,V> cacheEntry, JsonGenerator jsonGenerator,
			SerializerProvider serializerProvider) throws IOException,
			JsonProcessingException {
		jsonGenerator.writeStartObject();
		jsonGenerator.writeStringField("key", cacheEntry.getKey().toString());
		jsonGenerator.writeStringField("value", cacheEntry.getValue().toString());
		jsonGenerator.writeEndObject();
	}
	
}