package org.infinispan.quickstart.securecache.rest;

import java.security.Principal;
import java.security.PrivilegedAction;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Map;
import java.util.Set;

import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import javax.security.auth.Subject;
import javax.ws.rs.DELETE;
import javax.ws.rs.PUT;
import javax.ws.rs.Produces;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;

import org.infinispan.Cache;
import org.infinispan.manager.EmbeddedCacheManager;
import org.jboss.security.SecurityContextAssociation;

import javax.inject.Named;

@ApplicationScoped
@Path("/cache")
public class CacheRestService {

	@Inject
	private EmbeddedCacheManager cm;

	@GET
	@Path("/get")
	@Produces("application/json")
	public ArrayList<CacheEntry<String, String>> get(
			final @QueryParam("key") String key) {
		Subject subject = SecurityContextAssociation.getSubject();
		ArrayList<CacheEntry<String, String>> entries = Subject.doAs(subject,
				new PrivilegedAction<ArrayList<CacheEntry<String, String>>>() {
					@Override
					public ArrayList<CacheEntry<String, String>> run() {
						Cache<String, String> cache = cm.getCache("secured");
						ArrayList<CacheEntry<String, String>> cacheEntries = new ArrayList<CacheEntry<String, String>>();
						if (key == null) {
							Set<Map.Entry<String, String>> entries = cache
									.entrySet();
							for (Map.Entry<String, String> entry : entries) {
								cacheEntries
										.add(new CacheEntry<String, String>(
												entry.getKey(), entry
														.getValue()));
							}
						} else {
							String value = cache.get(key);
							if (value != null)
								cacheEntries
										.add(new CacheEntry<String, String>(
												key, value));
						}
						return cacheEntries;
					}
				});
		Collections.sort(entries);
		return entries;
	}

	@PUT
	@Path("/put")
	@Produces("application/json")
	public String put(final @QueryParam("key") String key,
			final @QueryParam("value") String value) {
		Subject subject = SecurityContextAssociation.getSubject();
		String entry = Subject.doAs(subject, new PrivilegedAction<String>() {
			@Override
			public String run() {
				Cache<String, String> cache = cm.getCache("secured");
				System.out.println(cache.getCacheConfiguration().security()
						.authorization().roles());
				return cache.putIfAbsent(key, value);
			}
		});
		return entry;
	}

	@GET
	@Path("/loggedUser")
	@Produces({ MediaType.TEXT_PLAIN })
	public String loggedUser() {
		Subject subject = SecurityContextAssociation.getSubject();
		Set<Principal> principals = subject.getPrincipals();
		String returnValue = "";
		for (Principal p : principals) {
			returnValue = returnValue.concat(p.getName() + " ");
		}
		return returnValue;
	}

	@DELETE
	@Path("/remove")
	@Produces("application/json")
	public Boolean remove(final @QueryParam("key") String key,
			final @QueryParam("value") String value) {
		Subject subject = SecurityContextAssociation.getSubject();
		Boolean entry = Subject.doAs(subject, new PrivilegedAction<Boolean>() {
			@Override
			public Boolean run() {
				Cache<String, String> cache = cm.getCache("secured");
				return cache.remove(key, value);
			}
		});
		return entry;
	}
}
